/**
 *
 */
package de.intevation.gnv.action;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.w3c.dom.Node;

import de.intevation.gnv.action.sessionmodel.SessionModel;
import de.intevation.gnv.action.sessionmodel.SessionModelFactory;
import de.intevation.gnv.artifactdatabase.client.ArtifactDatabaseClient;
import de.intevation.gnv.artifactdatabase.client.ArtifactDatabaseClientFactory;
import de.intevation.gnv.artifactdatabase.objects.ArtifactDescription;
import de.intevation.gnv.artifactdatabase.objects.ArtifactStatisticsSet;
import de.intevation.gnv.artifactdatabase.objects.DefaultInputParameter;
import de.intevation.gnv.artifactdatabase.objects.InputParameter;
import de.intevation.gnv.util.XMLUtils;
import de.intevation.gnv.util.XSLTransformer;

/**
 * @author Tim Englich <tim.englich@intevation.de>
 * 
 */
public class NextArtifactStepAction extends ArtifactDatabaseActionBase {
    /**
     * the logger, used to log exceptions and additonaly information
     */
    private static Logger log = Logger
            .getLogger(SelectArtifactFactoryAction.class);

    /**
     * Constructor
     */
    public NextArtifactStepAction() {
        super();
    }

    /**
     * @see de.intevation.gnv.action.ArtifactDatabaseActionBase#execute(org.apache.struts.action.ActionMapping,
     *      org.apache.struts.action.ActionForm,
     *      javax.servlet.http.HttpServletRequest,
     *      javax.servlet.http.HttpServletResponse)
     */
    @Override
    public ActionForward execute(ActionMapping mapping, ActionForm form,
                                 HttpServletRequest request,
                                 HttpServletResponse response) throws Exception {
            log.debug("NextArtifactStepAction.execute");
            SessionModel sm = SessionModelFactory.getInstance()
                    .getSessionModel(request);
            ArtifactDescription ad = sm.getArtifactDescription();
            if (ad != null){
                Collection<String> inputParameter = ad.getInputParameter();
                Collection<InputParameter> ips = null;
                if (inputParameter != null) {
                    ips = new ArrayList<InputParameter>(inputParameter.size());
                    Iterator<String> it = inputParameter.iterator();
                    while (it.hasNext()) {
                        String name = it.next();
                        String[] values = request.getParameterValues(name);
                        InputParameter ip = new DefaultInputParameter(name, values);
                        ips.add(ip);
                    }
                }
                ArtifactDatabaseClient adc = ArtifactDatabaseClientFactory
                                                    .getInstance()
                                                    .getArtifactDatabaseClient(getLocale(request));
                if (ad.getReachableStates() != null
                    && !ad.getReachableStates().isEmpty()) {
                    // TODO: Woher kommt der zu erreichende Status;
                    String target = null;
                    if (ad.getReachableStates().size() > 1) {
                        target = request.getParameter("product"); // TODO HACK for
                        // Propducts every
                        // other Step has
                        // currently only
                        // one reachable
                        // state.
                    } else {
                        target = ad.getReachableStates().iterator().next();
                    }
    
                    ArtifactDescription artifactDescription;
                    try {
                        artifactDescription = adc.doNextStep(sm.getSelectedArtifactFactory(), 
                                                             sm.getCurrentArtifact(),
                                                             target, ips);
                    } catch (Exception e) {
                        log.error(e, e);
                        request.setAttribute(
                                CommunicationKeys.REQUEST_EXCEPTION_MESSAGE_ID, e
                                        .getMessage());
                        artifactDescription = sm.getArtifactDescription();
                    }
    
                    Node currentUI = artifactDescription.getCurrentUI();
                    if (currentUI != null){
                        XSLTransformer transformer = new XSLTransformer();
                        Node dynamicUINode =  new XMLUtils().getNodeXPath(currentUI,"dynamic");
                        if (dynamicUINode != null){
                            String ui = transformer
                                    .transform(
                                            dynamicUINode,
                                            "UTF-8",
                                            request
                                                    .getRealPath("WEB-INF/config/templates/describe-ui.xsl"));
            
                            request.setAttribute("ui", ui);
                        }
                        
                        Node staticUINode = new XMLUtils().getNodeXPath(currentUI, "static");
                        if (staticUINode != null){
                            String staticUI = transformer
                                    .transform(
                                            staticUINode,
                                            "UTF-8",
                                            request
                                                    .getRealPath("WEB-INF/config/templates/describe-ui-static.xsl"));
                            request.setAttribute("staticui", staticUI);
                        }
                    }
                    request.setAttribute("furthertargets",
                            (ad.getReachableStates() != null && !ad
                                    .getReachableStates().isEmpty()));
    
                } else {
                    // statistic abholen und in das sessionmodell schreiben.
                    ArtifactDescription artifactDescription;
                    try{
                        
                        Collection<ArtifactStatisticsSet> statistics = adc
                                .calculateStatistics(sm.getSelectedArtifactFactory(),
                                        sm.getCurrentArtifact());
                        sm.setStatistics(statistics);
                        artifactDescription = adc
                                .getCurrentStepDescription(sm
                                        .getSelectedArtifactFactory(), sm
                                        .getCurrentArtifact(),
                                        true);
                        
                        request.setAttribute("diagramm", true);
                        
                    } catch (Exception e) {
                        log.error(e, e);
                        request.setAttribute(
                                CommunicationKeys.REQUEST_EXCEPTION_MESSAGE_ID, e
                                        .getMessage());
                        artifactDescription = sm.getArtifactDescription();
                    }
    
                    Node currentUI = artifactDescription.getCurrentUI();
                    if (currentUI != null){
                        XSLTransformer transformer = new XSLTransformer();
                        Node dynamicUINode =  new XMLUtils().getNodeXPath(currentUI,"dynamic");
                        if (dynamicUINode != null){
                            String ui = transformer
                                    .transform(
                                            dynamicUINode,
                                            "UTF-8",
                                            request
                                                    .getRealPath("WEB-INF/config/templates/describe-ui.xsl"));
            
                            request.setAttribute("ui", ui);
                        }
                        
                        Node staticUINode = new XMLUtils().getNodeXPath(currentUI, "static");
                        if (staticUINode != null){
                            String staticUI = transformer
                                    .transform(
                                            staticUINode,
                                            "UTF-8",
                                            request
                                                    .getRealPath("WEB-INF/config/templates/describe-ui-static.xsl"));
                            request.setAttribute("staticui", staticUI);
                        }
                    }
                    request.setAttribute("furthertargets", false);
                }
            }else{
                log.warn("SessionTimeout has occured");
                request.setAttribute(CommunicationKeys.REQUEST_EXCEPTION_MESSAGE_ID, 
                                     "SessionTimeout has occured");
                new FetchArtifactFactoriesAction().execute(mapping, form, request, response);
            }
            return super.execute(mapping, form, request, response);
    }

}
